#include "FireballParticleSystem.h"
#include "LineParticle.h"
#include "DirectionalBillboardParticle.h"
#include "Utils.h"
#include <math.h>
#include <iostream>

FireballParticleSystem::FireballParticleSystem(int numParticles) : ParticleSystem(numParticles) {
    GLuint texture;
    Utils::loadTexture("Textures/fireball.tga", &texture, GL_TEXTURE_2D, false);
    for (int i = 0; i < numParticles; i++) {   
        particles[i] = new DirectionalBillboardParticle(0, 0, 0, 0, 
                            1, 1, 1, 1, texture, 0.5,1,0,0,0);
    }   
    allowRenew = true;
}

FireballParticleSystem::~FireballParticleSystem() {
    for (int i = 0; i < numParticles; i++) {
        delete particles[i];    
    }   
}

void FireballParticleSystem::draw() {
    if (active) {
        glDisable(GL_LIGHTING);
        glEnable(GL_BLEND);
        glDepthMask(GL_FALSE);
        for (int i = 0; i < numParticles; i++) {
            if (particles[i]->isAlive()) {
                particles[i]->draw(); 
            } 
        }
        glDepthMask(GL_TRUE);
        glEnable(GL_LIGHTING);
        glDisable(GL_BLEND);        
    }
}

void FireballParticleSystem::idle(float deltaTime) {
    if (active) {
        bool someParticleAlive = false;
        for (int i = 0; i < numParticles; i++) {
            particles[i]->idle(deltaTime);     
            if (particles[i]->isAlive()) { \
                someParticleAlive = true;
                DirectionalBillboardParticle* ppart = (DirectionalBillboardParticle*) particles[i];
                ppart->setColorA(ppart->getLifetime() * 0.25);
            } else {        
                if (allowRenew) {
                    initParticle(particles[i]);
                    someParticleAlive = true;
                }
            }
        }     
        if (!someParticleAlive) {
            active = false;
        }
    }
}


void FireballParticleSystem::explode() {
    for (int i = 0; i < numParticles; i++) {
        allowRenew = false;
        DirectionalBillboardParticle* ppart = (DirectionalBillboardParticle*) particles[i];
        ppart->setVelocity(5);
    }
}

void FireballParticleSystem::initParticle(Particle* particle) {
    DirectionalBillboardParticle* ppart = (DirectionalBillboardParticle*) particle;
    ppart->setLifetime(Utils::random(0, 1));
    ppart->setAlive(true);
    ppart->setColorR(1);
    ppart->setColorG(1);
    ppart->setColorB(1);
    ppart->setColorA(0.25);
    ppart->setSize(Utils::random(0.3,0.5));
    
    float x = Utils::random(-0.5, 0.5);
    float y = Utils::random(-0.5, 0.5);
    float z = Utils::random(-0.5, 0.5);
    
    if ((x * x + y * y + z * z) <= 0.18) {
        ppart->setPosX(x);
        ppart->setPosY(y);
        ppart->setPosZ(z);
        ppart->setDirectionX(x);
        ppart->setDirectionY(y);
        ppart->setDirectionZ(z);
        ppart->setVelocity(0);
    } else {
        ppart->setAlive(false); //it's not in the sphere
    }
}

