#include "Hud.h"
#include <GL/glut.h>

Hud::Hud(Wizard* wizard) {
    this->wizard = wizard;
    Utils::loadTexture("Textures/hud-spray.tga", &texSpray, GL_TEXTURE_2D, false);
    Utils::loadTexture("Textures/hud-fireball.tga", &texBall, GL_TEXTURE_2D, false);
    Utils::loadTexture("Textures/hud-back.tga", &texBackground, GL_TEXTURE_2D, false);
    Utils::loadTexture("Textures/gradient.tga", &texGrad, GL_TEXTURE_2D, false);
}

void Hud::draw() {
    
    glEnable(GL_BLEND);
    drawBackground();
    drawSpells();
    glDisable(GL_BLEND);
    drawSpellLevels();
    drawBars();
    drawShop();
    if (!wizard->isAlive()) {
        glColor3f(1,1,1);
        Utils::renderString(200, 250, "GAME OVER!", GLUT_BITMAP_HELVETICA_18);
    }
}

void Hud::drawSpellLevels() {
    glColor3f(0,0,0);
    glPushMatrix();
    glRasterPos3f(30, 40, 0);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,wizard->getSprayLevel()+'0');
    glRasterPos3f(90, 40, 0);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,wizard->getFireballLevel()+'0');
    glPopMatrix();
    glColor3f(1,1,1);
}

void Hud::drawBackground() {
    glPushMatrix();
    glColor3f(1,1,1);
    glBindTexture(GL_TEXTURE_2D, texBackground);
    glBegin(GL_QUADS);
        glTexCoord2f(0,0);
        glVertex3f(0,0,0);
        glTexCoord2f(0,1);
        glVertex3f(0,100,0);
        glTexCoord2f(5,1);
        glVertex3f(500,100,0);
        glTexCoord2f(5,0);
        glVertex3f(500,0,0);
    glEnd();
    glPopMatrix();
    glBindTexture(GL_TEXTURE_2D, 0);
}

void Hud::drawSpells() {
    glColor3f(1,1,1);
    
    glPushMatrix();
    glBindTexture(GL_TEXTURE_2D, texSpray);
    glBegin(GL_QUADS);
        glTexCoord2f(0,0);
        glVertex2f(10,10);
        glTexCoord2f(0,1);
        glVertex2f(10,90);
        glTexCoord2f(1,1);
        glVertex2f(60,90);
        glTexCoord2f(1,0);
        glVertex2f(60,10);
    glEnd();
    glPopMatrix();
    
    glPushMatrix();
    glBindTexture(GL_TEXTURE_2D, texBall);
    glBegin(GL_QUADS);
        glTexCoord2f(0,0);
        glVertex2f(70,10);
        glTexCoord2f(0,1);
        glVertex2f(70,90);
        glTexCoord2f(1,1);
        glVertex2f(120,90);
        glTexCoord2f(1,0);
        glVertex2f(120,10);
    glEnd();
    glPopMatrix();
    glBindTexture(GL_TEXTURE_2D, 0);
}

void Hud::drawBars() {
    int health = wizard->getHealth();
    int mana = wizard->getMana();
    int gold = wizard->getGold();
    int maxHealth = wizard->getMaxHealth();
    glColor3f(1,0,0);
    glPushMatrix();
    glBegin(GL_LINES);
        glVertex2f(460,20);
        glVertex2f(480,20);
        glVertex2f(460,90);
        glVertex2f(480,90);
        glVertex2f(460,20);
        glVertex2f(460,90);
        glVertex2f(480,20);
        glVertex2f(480,90);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, texGrad);
    glBegin(GL_QUADS);
        glTexCoord2f(0,0);
        glVertex2f(460,20);
        glTexCoord2f(0,1);
        glVertex2f(460,20+70*health/((float) maxHealth));
        glTexCoord2f(1,1);
        glVertex2f(480,20+70*health/((float) maxHealth));
        glTexCoord2f(1,0);
        glVertex2f(480,20);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, 0);
    glPopMatrix();
    
    glColor3f(1,1,1);
    glRasterPos2f(465, 10);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,health/10+'0');
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,health%10+'0');
    
    glColor3f(0,0,1);
    glPushMatrix();
    glBegin(GL_LINES);
        glVertex2f(420,20);
        glVertex2f(440,20);
        glVertex2f(420,90);
        glVertex2f(440,90);
        glVertex2f(420,20);
        glVertex2f(420,90);
        glVertex2f(440,20);
        glVertex2f(440,90);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, texGrad);
    glBegin(GL_QUADS);
        glTexCoord2f(0,0);
        glVertex2f(420,20);
        glTexCoord2f(0,1);
        glVertex2f(420,20+70*mana/100.);
        glTexCoord2f(1,1);
        glVertex2f(440,20+70*mana/100.);
        glTexCoord2f(1,0);
        glVertex2f(440,20);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, 0);
    glPopMatrix();
    
    glColor3f(1,1,1);
    glRasterPos2f(425, 10);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,mana/10+'0');
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,mana%10+'0');
    
    glColor3f(1,1,0);
    glPushMatrix();
    glBegin(GL_LINES);
        glVertex2f(380,20);
        glVertex2f(400,20);
        glVertex2f(380,90);
        glVertex2f(400,90);
        glVertex2f(380,20);
        glVertex2f(380,90);
        glVertex2f(400,20);
        glVertex2f(400,90);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, texGrad);
    glBegin(GL_QUADS);
        glTexCoord2f(0,0);
        glVertex2f(380,20);
        glTexCoord2f(0,1);
        glVertex2f(380,20+70*gold/100.);
        glTexCoord2f(1,1);
        glVertex2f(400,20+70*gold/100.);
        glTexCoord2f(1,0);
        glVertex2f(400,20);
    glEnd();
    glBindTexture(GL_TEXTURE_2D, 0);
    
    glColor3f(1,1,1);
    glRasterPos2f(385, 10);
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,gold/10+'0');
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,gold%10+'0');
    glPopMatrix();
}

void Hud::drawShop() {
    glColor3f(1,1,1);
    glPushMatrix();
    glBegin(GL_LINES);
        glColor3f(0,1,1);
        glVertex2f(140,10);
        glVertex2f(240,10);
        glVertex2f(140,45);
        glVertex2f(240,45);
        glVertex2f(140,10);
        glVertex2f(140,45);
        glVertex2f(240,10);
        glVertex2f(240,45);
        
        glColor3f(1,0,1);
        glVertex2f(260,10);
        glVertex2f(360,10);
        glVertex2f(260,45);
        glVertex2f(360,45);
        glVertex2f(260,10);
        glVertex2f(260,45);
        glVertex2f(360,10);
        glVertex2f(360,45);
        
        glColor3f(1,0,0);
        glVertex2f(200,55);
        glVertex2f(300,55);
        glVertex2f(200,90);
        glVertex2f(300,90);
        glVertex2f(200,55);
        glVertex2f(200,90);
        glVertex2f(300,55);
        glVertex2f(300,90);
    glEnd();
    glPopMatrix();
    
    glPushMatrix();
    glColor3f(1,1,1);
    Utils::renderString(150, 20, "Upgrade Spray", GLUT_BITMAP_HELVETICA_10);
    Utils::renderString(270, 20, "Upgrade Fireball", GLUT_BITMAP_HELVETICA_10);
    Utils::renderString(210, 65, "Upgrade Health", GLUT_BITMAP_HELVETICA_10);
    glPopMatrix();
}

void Hud::handleMouse(int x, int y) {
    if (140 <= x && x <= 240 && 10 <= y && y <= 45) {
        wizard->upgradeSpray();
    } else if (260 <= x && x <= 360 && 10 <= y && y <= 45) {
        wizard->upgradeFireball();
    } else if (200 <= x && x <= 300 && 55 <= y && y <= 90) {
        wizard->upgradeHealth();
    }
}
