#include "Level.h"
#include "Floor.h"

Level::Level(Detector* detector, Wizard* wizard) {
    this->detector = detector;
    this->wizard = wizard;
    currLevel = 0;
    loadLevel1();
    switchLevel = false;
}

void Level::draw() {
    floor->draw();
    std::list<Wall*>::iterator i;
    std::list<Beast*>::iterator j;
    std::list<Room*>::iterator k;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        obj->draw();
    }
  
    for (j = beasts.begin(); j != beasts.end(); j++) {
        Beast* obj = *j;
        obj->draw();
    }
    for (k = rooms.begin(); k != rooms.end(); k++) {
        Room* obj = *k;   
        obj->draw();
    }    
}

void Level::drawForPick() {
    std::list<Room*>::iterator k;
    for (k = rooms.begin(); k != rooms.end(); k++) {
        Room* obj = *k;   
        obj->drawForPick();
    }
}

void Level::idle(float deltaTime) {
    if (switchLevel) {
        nextLevelReal();
    }
    std::list<Wall*>::iterator i;
    std::list<Beast*>::iterator j;
    std::list<Room*>::iterator k;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        obj->idle(deltaTime);
    }
    for (j = beasts.begin(); j != beasts.end(); ) {
        Beast* obj = *j;
        if (!obj->isAlive() && obj->isDeathAnimationDone()) {
            j++;
            delete obj;
        } else {
            obj->idle(deltaTime);
            j++;
        }
    }
    for (k = rooms.begin(); k != rooms.end(); k++) {
        Room* obj = *k;
        obj->idle(deltaTime);
    }
}

void Level::nextLevel() {
    switchLevel = true;
}

void Level::nextLevelReal() {
    std::list<Wall*>::iterator i;
    std::list<Beast*>::iterator j;
    std::list<Room*>::iterator k;
    for (i = walls.begin(); i != walls.end(); i++) {
        Wall* obj = *i;
        i--;
        walls.remove(obj);
        delete obj;
    }
    for (j = beasts.begin(); j != beasts.end(); j++) {
        Beast* obj = *j;
        j--;
        beasts.remove(obj);
        delete obj;
    }
    for (k = rooms.begin(); k != rooms.end(); k++) {
        Room* obj = *k;
        k--;
        rooms.remove(obj);
        delete obj;
    }
    delete floor;
    if (currLevel == 1) {
        loadLevel2();
    } else if (currLevel == 2) {
        loadLevel3();
    } else if (currLevel == 3) {
        loadLevel1();
    }
    switchLevel = false;
}

void Level::loadLevel1() {
    currLevel = 1;
    
    wizard->setPosition(10,0,10,135);
    
    floor = new Floor(BLACK_FLOOR);
    
    Wall* wall = new Wall(detector, BRICK_WALL);
    wall->setPosition(50,0,0,0);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, BRICK_WALL);
    wall->setPosition(50,0,100,0);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, BRICK_WALL);
    wall->setPosition(0,0,50,90);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, BRICK_WALL);
    wall->setPosition(100,0,50,90);
    wall->setLength(100);
    walls.push_back(wall);
    
    Room* room = new Room(detector,wizard,80,100,0,20,6,6,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,20,25,60,8,11,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,25,80,100,1,9,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,30,60,30,60,1,15,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,30,65,90,100,1,13,true,this, BRICK_WALL);
    rooms.push_back(room);

    room = new Room(detector,wizard,80,100,40,75,4,7,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,20,70,0,20,2,14,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,70,100,80,100,1,5,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,50,65,70,90,4,13,false,this, BRICK_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,20,40,4,4,false,this, BRICK_WALL);
    rooms.push_back(room);
}

void Level::loadLevel2() {
    currLevel = 2;
    
    wizard->setPosition(10,0,10,135);
    
    floor = new Floor(WOOD_FLOOR);
    
    Wall* wall = new Wall(detector, WOOD_WALL);
    wall->setPosition(50,0,0,0);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, WOOD_WALL);
    wall->setPosition(50,0,100,0);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, WOOD_WALL);
    wall->setPosition(0,0,50,90);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, WOOD_WALL);
    wall->setPosition(100,0,50,90);
    wall->setLength(100);
    walls.push_back(wall);
    
    Room* room = new Room(detector,wizard,60,100,0,20,4,6,false,this, WOOD_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,20,20,50,11,11,false,this, WOOD_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,30,50,30,50,15,15,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,50,70,30,50,15,11,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,30,50,50,70,15,14,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,50,70,50,70,15,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,90,100,30,50,5,7,false,this, WOOD_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,90,100,50,70,6,6,false,this, WOOD_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,15,60,100,9,9,false,this, WOOD_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,25,70,80,100,1,13,true,this, WOOD_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,80,100,5,5,false,this, WOOD_WALL);
    rooms.push_back(room);
}

void Level::loadLevel3() {
    currLevel = 3;
    
    wizard->setPosition(10,0,10,135);
    
    floor = new Floor(GRASS_FLOOR);
    
    Wall* wall = new Wall(detector, WHITE_WALL);
    wall->setPosition(50,0,0,0);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, WHITE_WALL);
    wall->setPosition(50,0,100,0);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, WHITE_WALL);
    wall->setPosition(0,0,50,90);
    wall->setLength(100);
    walls.push_back(wall);
    
    wall = new Wall(detector, WHITE_WALL);
    wall->setPosition(100,0,50,90);
    wall->setLength(100);
    walls.push_back(wall);
    
    Room* room = new Room(detector,wizard,20,40,0,20,12,14,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,40,60,0,20,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,60,80,0,20,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,0,20,6,2,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,20,20,40,10,11,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,20,40,20,40,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,40,60,20,40,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,60,80,20,40,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,20,40,5,2,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,20,40,60,9,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,20,40,40,60,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,40,60,40,60,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,60,80,40,60,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,40,60,6,2,false,this, WHITE_WALL);
    rooms.push_back(room);    
    
    room = new Room(detector,wizard,0,20,60,80,10,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,20,40,60,80,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,40,60,60,80,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,60,80,60,80,12,10,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,60,80,5,2,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,0,20,80,100,9,8,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,20,40,80,100,12,8,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,40,60,80,100,12,8,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,60,80,80,100,12,8,false,this, WHITE_WALL);
    rooms.push_back(room);
    
    room = new Room(detector,wizard,80,100,80,100,4,0,true,this, WHITE_WALL);
    rooms.push_back(room);
}
