#include "MeltingModelParticleSystem.h"
#include "BillboardParticle.h"
#include "Utils.h"

MeltingModelParticleSystem::MeltingModelParticleSystem(Model* mod, float dropSize,
                       GLfloat r, GLfloat g, GLfloat b) : ParticleSystem(mod->getModelNumVertices()) {
    this->model = mod;
    velocity = 0;
    this->r = r;
    this->g = g;
    this->b = b;
    GLuint texture;
    Utils::loadTexture("Textures/circle.tga", &texture, GL_TEXTURE_2D, false);
    coord* vertices = model->getModelVertices();
    for (int i = 0; i < model->getModelNumVertices(); i++) {   
          particles[i] = new BillboardParticle(0, 
                            vertices[i].x, vertices[i].y, vertices[i].z, 
                            r, g, b, 1, texture, dropSize);
    }   
}

MeltingModelParticleSystem::~MeltingModelParticleSystem() {
    for (int i = 0; i < numParticles; i++) {
        delete particles[i];    
    }   
}

void MeltingModelParticleSystem::idle(float deltaTime) {
    if (active) {
        bool someParticleAlive = false;
        for (int i = 0; i < numParticles; i++) {
            particles[i]->idle(deltaTime);
            velocity -= 0.1 * deltaTime;
            if (particles[i]->isAlive()) {    
                someParticleAlive = true;        
                GLfloat y = particles[i]->getPosY();
                y = y + velocity * deltaTime;
                if (y > 0 - model->getPositionCorrectionY()) {
                    particles[i]->setPosY(y);
                } else {
                    particles[i]->setPosY(0 - model->getPositionCorrectionY());
                }
            }
        }     
        if (!someParticleAlive) {
            active = false;    
        }
    }
}

void MeltingModelParticleSystem::draw() {
    if (active) {
        glDisable(GL_LIGHTING);
        glEnable(GL_BLEND);
        glDepthMask(GL_FALSE);
        glPushMatrix();
        glScalef(model->getScale(),model->getScale(),model->getScale());
        glTranslatef(model->getPositionCorrectionX(),model->getPositionCorrectionY(),model->getPositionCorrectionZ());
        glRotatef(model->getHeadingCorrection(), 0,1,0);
        for (int i = 0; i < numParticles; i++) {
            if (particles[i]->isAlive()) {
                particles[i]->draw();   
            } 
        }
        glPopMatrix();
        glDepthMask(GL_TRUE);
        glEnable(GL_LIGHTING);
        glDisable(GL_BLEND);        
    }
}

void MeltingModelParticleSystem::reset() {
    velocity = 0;
    coord* vertices = model->getModelVertices();
    for (int i = 0; i < numParticles; i++) {   
        BillboardParticle* ppart = (BillboardParticle*) particles[i]; //this is ok
        ppart->setLifetime(2);
        ppart->setAlive(true);
        ppart->setColorR(r);
        ppart->setColorG(g);
        ppart->setColorB(b);
        ppart->setColorA(1);
        ppart->setPosX(vertices[i].x);
        ppart->setPosY(vertices[i].y);
        ppart->setPosZ(vertices[i].z);   
    }   
}
