#include "Model.h"
#include "Utils.h"

Model::Model() {
    positionCorrection[0] = 0;
    positionCorrection[1] = 0;
    positionCorrection[2] = 0;
    headingCorrection = 0;
    animationActive = false;
    numAnimations = 0;
    currentAnimation = 0;
    textureId = 0;
    scale = 1;
}

Model::Model(std::string modelFileName, std::string textureFileName) {
    char tex[30];
    memset(tex, '\0', 30);
    textureFileName.copy(tex,textureFileName.length());
    char mdl[30];
    memset(mdl, '\0', 30);
    modelFileName.copy(mdl,modelFileName.length());
    Utils::loadTexture(tex, &textureId, GL_TEXTURE_2D, false);
    object.loadObj(mdl);
    float x1,x2,x3,x4,x5,x6;
    object.getBoundingVolume(&x1,&x2,&x3,&x4,&x5,&x6);
    positionCorrection[0] = 0;
    positionCorrection[1] = -x3;
    positionCorrection[2] = 0;
    headingCorrection = 0;
    numAnimations = 0;
    currentAnimation = 0;
    scale = 1;
}

void Model::setScale(float s) {
    scale = s;
}

void Model::setPositionCorrection(float x, float y, float z) {
    positionCorrection[0] = x;
    positionCorrection[1] = y;
    positionCorrection[2] = z;
}

void Model::setHeadingCorrection(float h) {
    headingCorrection = h;
}

coord* Model::getModelVertices() {
    return object.getVertexList();
}

int Model::getModelNumVertices() {
    return object.getVertexCount();
}

void Model::addAnimation(int animId, Animation* animation) {
    animations[animId] = animation;
    numAnimations++;
}

void Model::removeAnimation(int animId) {
    animations.erase(animId);
    numAnimations--;
}

void Model::setCurrentAnimation(int animId) {
    if (numAnimations > 0) {
        if (animationActive) {
            currentAnimation = animId;
            startAnimation();
        } else {
            currentAnimation = animId;
        }
    }
}

void Model::startAnimation() {
    if (numAnimations > 0) {
        animations[currentAnimation]->reset();
        animationActive = true;
    }
}

void Model::stopAnimation() {
    if (numAnimations > 0) {
        animations[currentAnimation]->stopGracefully();
    }
}

bool Model::isAnimating() {
    return animationActive;    
}

int Model::getCurrentAnimation() {
    return currentAnimation;   
}

void Model::idle(float deltaTime) {
    if (numAnimations > 0 && animationActive) {
        if (animations.find(currentAnimation) != animations.end()) {
            Animation* currentAnim = animations[currentAnimation];
            if (currentAnim->isActive()) {
                currentAnim->animate(deltaTime);
            } else {
                animationActive = false;
            }
        } else {
            //This is because of programmer error!!!!!!!
            exit(2);
        }
    }
}

void Model::draw() {
    glPushMatrix();
        glScalef(scale,scale,scale);
        glTranslatef(positionCorrection[0],positionCorrection[1],positionCorrection[2]);
        glRotatef(headingCorrection, 0,1,0);
        glBindTexture(GL_TEXTURE_2D, textureId);
        object.displayObj();
    glPopMatrix();
    
//    glPushMatrix();
//    float minX,maxX,minY,maxY,minZ,maxZ;
//    getBoundingVolume(&minX,&maxX,&minY,&maxY,&minZ,&maxZ);
//    glScalef(maxX-minX,maxY-minY,maxZ-minZ);
//    glutSolidCube(1);
//    glPopMatrix();
}

void Model::getBoundingVolume(float *minX,float *maxX,
                                float *minY,float *maxY,
                                float *minZ,float *maxZ) {
    object.getBoundingVolume(minX,maxX,minY,maxY,minZ,maxZ);
//    *minX = *minX * scale + 0.1;
//    *maxX = *maxX * scale - 0.1;
//    *minY = *minY * scale + 0.1;
//    *maxY = *maxY * scale - 0.1;
//    *minZ = *minZ * scale + 0.1;
//    *maxZ = *maxZ * scale - 0.1;
    *minX = *minX * scale;
    *maxX = *maxX * scale;
    *minY = *minY * scale;
    *maxY = *maxY * scale;
    *minZ = *minZ * scale;
    *maxZ = *maxZ * scale;
}
