#include "RainParticleSystem.h"
#include "LineParticle.h"
#include "BillboardParticle.h"
#include "Utils.h"

RainParticleSystem::RainParticleSystem(int numParticles, float dropSize, float lifetimeLower, float lifetimeUpper, float fallRate,
                       GLfloat r, GLfloat g, GLfloat b,
                       GLfloat top, GLfloat bottom, GLfloat left, 
                       GLfloat right, GLfloat front, GLfloat back) : ParticleSystem(numParticles) {
    this->top = top;
    this->bottom = bottom;
    this->left = left;
    this->right = right;
    this->back = back;
    this->front = front;
    this->lifetimeUpper = lifetimeUpper;
    this->lifetimeLower = lifetimeLower;
    this->fallRate = fallRate;
    this->r = r;
    this->g = g;
    this->b = b;
    GLuint texture;
    Utils::loadTexture("Textures/circle.tga", &texture, GL_TEXTURE_2D, false);
    for (int i = 0; i < numParticles; i++) {        
          particles[i] = new BillboardParticle(Utils::random(0,1), 
                            Utils::random(left, right),
                            Utils::random(bottom,top),
                            Utils::random(back, front), 
                            r, g, b, 1, texture, dropSize);
    }   
}

RainParticleSystem::~RainParticleSystem() {
    for (int i = 0; i < numParticles; i++) {
        delete particles[i];    
    }   
}

void RainParticleSystem::idle(float deltaTime) {
    if (active) {
        for (int i = 0; i < numParticles; i++) {
            particles[i]->idle(deltaTime);
            
            if (particles[i]->isAlive()) {            
                GLfloat y = particles[i]->getPosY();
                y = y - (fallRate * deltaTime);
                if (y > bottom) {
                    particles[i]->setPosY(y);
                } else {
                    particles[i]->setAlive(false);
                }
            } else {            
                BillboardParticle* ppart = (BillboardParticle*) particles[i]; //this is ok
                ppart->setLifetime(Utils::random(lifetimeLower, lifetimeUpper));
                ppart->setAlive(true);
                ppart->setColorR(r);
                ppart->setColorG(g);
                ppart->setColorB(b);
                ppart->setColorA(1);
                ppart->setPosX(Utils::random(left, right));
                ppart->setPosY(top);
                ppart->setPosZ(Utils::random(front, back));
            }
        }     
    }
}
