#include "Skybox.h"
#include "Utils.h"

Skybox::Skybox(char* frontTexture, char* backTexture, char* leftTexture, char* rightTexture, char* upTexture, char* downTexture) {
    Utils::loadTexture(frontTexture, &frontTex, GL_TEXTURE_2D, true);  
    Utils::loadTexture(backTexture, &backTex, GL_TEXTURE_2D, true); 
    Utils::loadTexture(leftTexture, &leftTex, GL_TEXTURE_2D, true); 
    Utils::loadTexture(rightTexture, &rightTex, GL_TEXTURE_2D, true); 
    Utils::loadTexture(upTexture, &upTex, GL_TEXTURE_2D, true); 
    Utils::loadTexture(downTexture, &downTex, GL_TEXTURE_2D, true);  
    
    xRot = 0;
    yRot = 0;
    zRot = 0;    
}

void Skybox::draw() {
    glPushMatrix();
    
    glRotatef(xRot,1,0,0);
    glRotatef(yRot,0,1,0);
    glRotatef(zRot,0,0,1);
    
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    
    glBindTexture(GL_TEXTURE_2D, frontTex);
    glBegin(GL_QUADS); //Front
        glTexCoord2f(0.0, 0.0); 
        glVertex3f(-1.0, -1.0,  1.0);  
        glTexCoord2f(1.0, 0.0); 
        glVertex3f( 1.0, -1.0,  1.0);  
        glTexCoord2f(1.0, 1.0); 
        glVertex3f( 1.0,  1.0,  1.0);  
        glTexCoord2f(0.0, 1.0); 
        glVertex3f(-1.0,  1.0,  1.0);
    glEnd();    
    
    glBindTexture(GL_TEXTURE_2D, backTex);
    glBegin(GL_QUADS); //Back
        glTexCoord2f(1.0, 0.0); 
        glVertex3f(-1.0, -1.0, -1.0);  
        glTexCoord2f(1.0, 1.0); 
        glVertex3f(-1.0,  1.0, -1.0);  
        glTexCoord2f(0.0, 1.0); 
        glVertex3f( 1.0,  1.0, -1.0);  
        glTexCoord2f(0.0, 0.0); 
        glVertex3f( 1.0, -1.0, -1.0);
    glEnd();        
        
    glBindTexture(GL_TEXTURE_2D, upTex);
    glBegin(GL_QUADS); //Top
        glTexCoord2f(0.0, 1.0); 
        glVertex3f(-1.0,  1.0, -1.0);  
        glTexCoord2f(0.0, 0.0); 
        glVertex3f(-1.0,  1.0,  1.0);  
        glTexCoord2f(1.0, 0.0); 
        glVertex3f( 1.0,  1.0,  1.0);
        glTexCoord2f(1.0, 1.0); 
        glVertex3f( 1.0,  1.0, -1.0);
    glEnd();
    
    glBindTexture(GL_TEXTURE_2D, downTex);
    glBegin(GL_QUADS); //Bottom      
        glTexCoord2f(0.0, 0.0); 
        glVertex3f(-1.0, -1.0, -1.0);  
        glTexCoord2f(1.0, 0.0); 
        glVertex3f( 1.0, -1.0, -1.0);  
        glTexCoord2f(1.0, 1.0); 
        glVertex3f( 1.0, -1.0,  1.0);  
        glTexCoord2f(0.0, 1.0); 
        glVertex3f(-1.0, -1.0,  1.0);  
    glEnd();
        
    glBindTexture(GL_TEXTURE_2D, rightTex);
    glBegin(GL_QUADS); //Right
        glTexCoord2f(1.0, 0.0); 
        glVertex3f( 1.0, -1.0, -1.0); 
        glTexCoord2f(1.0, 1.0); 
        glVertex3f( 1.0,  1.0, -1.0);
        glTexCoord2f(0.0, 1.0); 
        glVertex3f( 1.0,  1.0,  1.0);
        glTexCoord2f(0.0, 0.0); 
        glVertex3f( 1.0, -1.0,  1.0);
    glEnd();

    glBindTexture(GL_TEXTURE_2D, leftTex);
    glBegin(GL_QUADS); //Left
        glTexCoord2f(0.0, 0.0); 
        glVertex3f(-1.0, -1.0, -1.0);
        glTexCoord2f(1.0, 0.0); 
        glVertex3f(-1.0, -1.0,  1.0);
        glTexCoord2f(1.0, 1.0); 
        glVertex3f(-1.0,  1.0,  1.0);
        glTexCoord2f(0.0, 1.0); 
        glVertex3f(-1.0,  1.0, -1.0);
        glEnd();
    glBindTexture(GL_TEXTURE_2D,0);
    
    glEnable(GL_LIGHTING);
    glEnable(GL_DEPTH_TEST);    
     
    glPopMatrix();
}
