#include "Spray.h"
#include "Wizard.h"
#include <math.h>

Spray::Spray(Detector* detector, int power, Wizard* wizard) : Spell(detector, power) {
    this->wizard = wizard;
    particles = new SprayParticleSystem(100);
    particles->setActive(true);
}

Spray::~Spray() {
    detector->removeObject(this);
    delete particles;
}
    
    
void Spray::idle(float deltaTime) {
    particles->idle(deltaTime);
    
    orientation = wizard->getOrientation();
    positionX = cos(-orientation/180.0*M_PI) * -1.6 - sin(-orientation/180.0*M_PI) * -0.3;
    positionY = 0.8;
    positionZ = sin(-orientation/180.0*M_PI) * -1.6 + cos(-orientation/180.0*M_PI) * -0.3;     
    
    positionX += wizard->getPositionX();
    positionY += wizard->getPositionY();
    positionZ += wizard->getPositionZ();
    
            
}

void Spray::draw() {
    glPushMatrix();    
    glTranslatef(wizard->getPositionX(), wizard->getPositionY(), wizard->getPositionZ());
    glRotatef(wizard->getOrientation(),0,1,0);
    particles->draw();
    glPopMatrix(); 
}

void Spray::getBoundingVolume(float *minX,float *maxX,
                   float *minY,float *maxY,
                   float *minZ,float *maxZ) {
    *minX = -1;
    *maxX = 1;
    *minY = -1;
    *maxY = 1;
    *minZ = -1;
    *maxZ = 1;                
}

int Spray::getStrength() {
    return power;
}
