#include "SprayParticleSystem.h"
#include "DirectionalBillboardParticle.h"
#include "Utils.h"
#include <math.h>
#include <iostream>

SprayParticleSystem::SprayParticleSystem(int numParticles) : ParticleSystem(numParticles) {
    GLuint texture;
    Utils::loadTexture("Textures/star.tga", &texture, GL_TEXTURE_2D, false);
    for (int i = 0; i < numParticles; i++) {   
        particles[i] = new DirectionalBillboardParticle(0, 0, 0, 0, 
                           1, 1, 1, 1, texture, 1,1,0,0,1);
    }   
}

SprayParticleSystem::~SprayParticleSystem() {
    for (int i = 0; i < numParticles; i++) {
        delete particles[i];    
    }   
}

void SprayParticleSystem::idle(float deltaTime) {
    if (active) {
        for (int i = 0; i < numParticles; i++) {
            particles[i]->idle(deltaTime);
            if (particles[i]->isAlive()) {      
                DirectionalBillboardParticle* ppart = (DirectionalBillboardParticle*) particles[i]; //this is ok
                ppart->setColorA(ppart->getLifetime());
            } else {          
                initParticle(particles[i]);
            }
        }     
    }
}

void SprayParticleSystem::initParticle(Particle* p) {
    DirectionalBillboardParticle* ppart = (DirectionalBillboardParticle*) p; //this is ok
    float r,g,b;
    if (Utils::random(0,1) > 0.5) {
        r = 0.5;
        b = 1;
        g = 0.5;
    } else {
        r = 0.5;
        b = 0.5;
        g = 1;
    }
    ppart->setLifetime(Utils::random(0, 1));
    ppart->setAlive(true);
    ppart->setColorR(r);
    ppart->setColorG(g);
    ppart->setColorB(b);
    ppart->setColorA(ppart->getLifetime());
    ppart->setSize(Utils::random(0.1,0.2));
    ppart->setVelocity(Utils::random(3,5));
    ppart->setDirectionX(-2);
    ppart->setDirectionY(Utils::random(-0.25,0.25));
    ppart->setDirectionZ(Utils::random(-0.25,0.25));
    ppart->setPosX(-0.6);
    ppart->setPosY(0.8);
    ppart->setPosZ(-0.3);
}

void SprayParticleSystem::reset() {
    for (int i = 0; i < numParticles; i++) {
        initParticle(particles[i]);
    }   
}
