#include "Utils.h"
#include "ltga.h"
#include <stdlib.h>

std::map<std::string, GLuint> Utils::textures = std::map<std::string, GLuint>();

void Utils::loadTexture(std::string name, GLuint* dest, GLenum type, bool clamp) {
    if (textures.find(name) != textures.end()) {
        *dest = textures[name];
        return;
    }    
    
    LTGA tga;
    glGenTextures(1, dest);   
    glBindTexture(type, *dest);
    tga.LoadFromFile(name);
    glTexParameteri(type, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    glTexParameteri(type, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);  
    if (clamp) {
        glTexParameteri(type, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(type, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    }
    GLenum mode;
    GLenum components;
    if (tga.GetImageType() == itRGBA) {
        mode = GL_RGBA;
        components = GL_RGBA;
    } else {
        mode = GL_RGB;   
        components = GL_RGB8;
    }
    gluBuild2DMipmaps(type, components, tga.GetImageWidth(), tga.GetImageHeight(), mode, GL_UNSIGNED_BYTE, tga.GetPixels());
    textures[name] = *dest;
}

float Utils::random(float lowerBound, float upperBound) {
    return lowerBound + rand() / (RAND_MAX + 1.0) * (upperBound - lowerBound);
}

int Utils::randomInt(int lowerBound, int upperBound) {
    return lowerBound + rand() % (upperBound - lowerBound + 1);
}

float Utils::interpVal(float desired, float current, float deltaTime, float timeLeft) {
    float distanceLeft = desired - current;
    if (distanceLeft < 0) distanceLeft = -distanceLeft;
    if (current < desired) {
        return current + distanceLeft * deltaTime / timeLeft;
    } else {
        return current - distanceLeft * deltaTime / timeLeft;
    }
}

void Utils::renderString(int x, int y, char* string, void* font) {
    int len, i;
    glRasterPos2f(x, y);
    len = (int) strlen(string);
    for (i = 0; i < len; i++) {
        glutBitmapCharacter(font, string[i]);
    }
}

void Utils::renderString(float x, float y, float z, char* string, void* font) {
    glDisable(GL_LIGHTING);
    int len, i;
    glRasterPos3f(x, y, z);
    len = (int) strlen(string);
    for (i = 0; i < len; i++) {
        glutBitmapCharacter(font, string[i]);
    }
    glEnable(GL_LIGHTING);
}
