#ifndef _WIZARD_H_
#define _WIZARD_H_

#include "Object.h"
#include "Gold.h"
#include "Fireball.h"
#include "Spray.h"
#include "Enemy.h"

#define WIZARD_REST 10001
#define WIZARD_WALK 10002
#define WIZARD_CAST 10003
#define WIZARD_DEAD 10004

class Wizard : public Object {
    
private:

    int health;
    int maxHealth;
    int mana;
    int maxMana;
    int gold;
    std::list<Fireball*> fireballs;
    
    Spray* spray;
    float sprayTime;
    
    int fireballLevel;
    int sprayLevel;
    float regenerateHTime;
    float regenerateMTime;
    bool animating;
    bool alive;
    
public:

    Wizard(Detector* detector);
    
    virtual void moveForward(float deltaTime);
    virtual void moveBackward(float deltaTime);
    void castAnimation();
    void stop();
    void changeHealth(int amount);
    void die();
    void pickup(Gold* gold);
    int getHealth();
    int getMana();
    int getGold();
    void upgradeSpray();
    void upgradeFireball();
    void upgradeHealth();
    void idle(float deltaTime);
    int getMaxHealth();
    void castFireball(Enemy* beast);
    void castSpray();
    
    void draw();
    void removeFireball(Fireball* fireball);
    int getSprayLevel();
    int getFireballLevel();
    bool isAnimating();
    void setAnimating(bool anim);
    bool isAlive();
    void getBoundingVolume(float *minX,float *maxX,
                           float *minY,float *maxY,
                           float *minZ,float *maxZ);
    
    

};

#endif //_WIZARD_H_
